brand.ratings <- read.csv("http://goo.gl/IQl8nc")
head(brand.ratings)

str(brand.ratings)
library(psych)
describe(brand.ratings)
brand.sc <- brand.ratings
brand.sc[, 1:9] = apply(brand.ratings[, 1:9], 2, scale)
summary(brand.sc)

pairs.panels(brand.sc[, 1:9])
library(corrplot)
corrplot(cor(brand.sc[, 1:9]), order = "hclust")


brand.pc <- prcomp(brand.sc[, 1:9])
summary(brand.pc)
# compute mean for heatmap interpretation
adj.mean <- aggregate(.~brand, data = brand.ratings, mean)
rownames(adj.mean) <- adj.mean[,1]
adj.mean <- adj.mean[, -1]
adj.mean
biplot(adj.mean) #, main = "Brand positioning")

brand.mean <- aggregate(brand.sc[,1:9], list(brand.sc[,10]), mean)
rownames(brand.mean) = brand.mean[, 1]
brand.mu.pc <- prcomp(brand.mean[, 2:10], scale = TRUE)
summary(brand.mu.pc)
biplot(brand.mu.pc, main = "Brand positioning")

brand.mean <- aggregate(. ~ brand, data = brand.sc, mean)
brand.mean[3, -1] - brand.mean[5, -1]


######################################################################################
######################################################################################
######################################################################################
######################################################################################
library(tidyverse)
library(psych)
library(corrplot)
#load data
data("USArrests")

#view first six rows of data
head(USArrests)
corrplot::corrplot(cor(USArrests), order = "hclust")
apply(USArrests, 2, mean)
apply(USArrests, 2, sd)

#calculate principal components
results <- prcomp(USArrests, scale = TRUE,retx=TRUE)
summary(results)

#display the first six scores
head(results$x)

#display states with highest murder rates in original dataset
head(USArrests[order(-USArrests$Murder),])

#calculate total variance explained by each principal component
results$sdev^2 / sum(results$sdev^2)

#calculate total variance explained by each principal component
var_explained = results$sdev^2 / sum(results$sdev^2)

#Check if Eigen vectors are normalized
apply((results$rotation)^2, 2, sum)


#create scree plot
qplot(c(1:4), var_explained) + 
  geom_line() + 
  xlab("Principal Component") + 
  ylab("Variance Explained") +
  ggtitle("Scree Plot") +
  ylim(0, 1)

qplot(c(1:4), cumsum(var_explained)) + 
  geom_line() + 
  xlab("Principal Component") + 
  ylab("Variance Explained") +
  ggtitle("Scree Plot") +
  ylim(0, 1)

cumsum(var_explained)
choose.PC<-min(which(cumsum(var_explained)>=0.8))
choose.PC

#On checking the weights of 2 principal components, we see that:
results$rotation

#The first loading vector places approximately equal weight on Assault, Murder, and Rape, 
#with much less weight on UrbanPop. Hence this component roughly corresponds to a measure of overall rates of serious crimes.
#The second loading vector places most of its weight on UrbanPop and much less weight on 
#the other three features. Hence, this component roughly corresponds to the level of urbanization of the state.


results$rotation=-results$rotation
results$x=-results$x
biplot (results , scale =0)

#The below biplot shows that 50 states mapped according to the 2 principal components. 
#The vectors of the PCA for 4 variables are also plotted.

#The large positive scores on the first component, such as California, Nevada and Florida, 
#have high crime rates, while states like North Dakota, with negative scores on the first component, 
#have low crime rates.
#California also has a high score on the second component, indicating a high level of urbanization, 
#while the opposite is true for states like Mississippi.
#States close to zero on both components, such as Indiana, have approximately average 
#levels of both crime and urbanization.


####################################################################
####################################################################
####################### Step by spet PCA ###########################
####################################################################
####################################################################

X<-as.matrix(USArrests)
d<-dim(X)[2]
mu<-apply(X, 2, mean)
U<-NULL
for (j in 1:d) U<-cbind(U,X[,j]-mu[j])
n<-dim(U)[1]
S<-(t(U)%*%U)/(n-1)
round(S, digits = 10)

#Alternatively
S<-cov(X)

Sigma<-S

Eigen<-eigen(Sigma)
Phi<-Eigen$vectors
lambda<-Eigen$values
round(t(Phi)%*%Sigma%*%Phi, digits = 10)
diag(lambda)

#Check: Phi%*%diag(lambda)%*%t(Phi)=Sigma
round(Phi%*%diag(lambda)%*%t(Phi), digits = 10)
Sigma


r<-(lambda/sum(lambda))

var_explained
r

sig<-apply(X, 2, sd)

#### Comparing results obtained with prcomp and results obtained manually

results$sdev
sqrt(lambda)

results$rotation
Phi

results$center
apply(X, 2, mean)

results$scale
apply(X, 2, sd)

############################################################################################
############################################################################################
####################### Step by spet PCA with correlation matrix ###########################
############################################################################################
############################################################################################

X<-as.matrix(USArrests)
d<-dim(X)[2]
mu<-apply(X, 2, mean)
U<-NULL
for (j in 1:d) U<-cbind(U,X[,j]-mu[j])
n<-dim(X)[1]
S<-(t(U)%*%U)/(n-1)
c2c<-function(S){
  C<-matrix(NA,nrow=dim(S)[1], ncol = dim(S)[2])
  for (i in 1:dim(S)[1]){
    for (j in 1:dim(S)[2]) {
      C[i,j]=S[i,j]/(sqrt(S[i,i])*sqrt(S[j,j]))
    } 
  }
  return(C)
}
C<-c2c(S)

#Alternatively
C<-cov2cor(S)

#Alternatively
C<-cor(X)

Eigen<-eigen(C)
Phi<-Eigen$vectors

lambda<-Eigen$values
round(t(Phi)%*%C%*%Phi, digits = 10)
diag(lambda)

#Check: Phi%*%diag(lambda)%*%t(Phi)=C
round(Phi%*%diag(lambda)%*%t(Phi), digits = 10)
C


r<-(lambda/sum(lambda))

r
var_explained

#### Comparing results obtained with prcomp and results obtained manually

results$sdev
sqrt(lambda)

results$rotation
Phi

results$center
apply(X, 2, mean)

results$scale
apply(X, 2, sd)

biplot()

##################################################################
##################################################################
##################################################################

#*mpg: Fuel consumption (Miles per (US) gallon): more powerful and heavier cars tend to consume more fuel. 
#*cyl: Number of cylinders: more powerful cars often have more cylinders 
#*disp: Displacement (cu.in.): the combined volume of the engine's cylinders 
#*hp: Gross horsepower: this is a measure of the power generated by the car 
#*drat: Rear axle ratio: this describes how a turn of the drive shaft corresponds to a turn of the wheels. Higher values will decrease fuel efficiency.
#*wt: Weight (1000 lbs): pretty self-explanatory! 
#*qsec: 1/4 mile time: the cars speed and acceleration 
#*vs: Engine block: this denotes whether the vehicle's engine is shaped like a "V", or is a more common straight shape. 
#*am: Transmission: this denotes whether the car's transmission is automatic (0) or manual (1). 
#*gear: Number of forward gears: sports cars tend to have more gears. *carb: Number of carburetors: associated with more powerful engines

head(mtcars)

res <- cor(mtcars, method="pearson")
corrplot::corrplot(res, method= "color", order = "hclust", tl.pos = 'n')

mtcars.pca <- prcomp(mtcars[,c(1:7,10,11)], center = TRUE,scale. = TRUE)
summary(mtcars.pca)

## Data visualization: the ggbiplot package
library(devtools)
install_github("vqv/ggbiplot")

### A biplot includes both the position of each sample in terms of PC1 and PC2 and 
# also will show you how the initial variables map onto this


library(ggbiplot)

ggbiplot(mtcars.pca)

#Add the car model to the plot
ggbiplot(mtcars.pca, labels=rownames(mtcars))

#create a vector containing the Country where cars are manufactured
mtcars.country <- c(rep("Japan", 3), rep("US",4), rep("Europe", 7),rep("US",3), "Europe", rep("Japan", 3), rep("US",4), rep("Europe", 3), "US", rep("Europe", 3))

ggbiplot(mtcars.pca,ellipse=TRUE,  labels=rownames(mtcars), groups=mtcars.country)


#Now you see something interesting: the US cars form a distinct cluster to the right. 
#Looking at the axes, you see that the US cars are characterized by high values for cyl,
#disp, and wt. Japanese cars, on the other hand, are characterized by high mpg. European cars 
#are somewhat in the middle and less tightly clustered than either group. Of course, you have
#many principal components available, each of which map differently to the original variables. 
#You can also ask ggbiplot to plot these other components, by using the choices argument.

#If you want to visualize the third and fourth components:
ggbiplot(mtcars.pca,ellipse=TRUE,choices=c(3,4),   labels=rownames(mtcars), groups=mtcars.country)
